@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2012-2013, Benjamin Drung <bdrung@debian.org>
+ * Copyright (C) 2012-2014, Benjamin Drung <bdrung@debian.org>
  *
  * Permission to use, copy, modify, and/or distribute this software for any
  * purpose with or without fee is hereby granted, provided that the above
@@ -38,8 +38,13 @@
 static char *milestones[] = {"created"
                             ,"release"
                             ,"eol"
+#ifdef DEBIAN
+                            ,"eol-lts"
+                            ,"eol-elts"
+#endif
 #ifdef UBUNTU
                             ,"eol-server"
+                            ,"eol-esm"
 #endif
 };
 
@@ -102,7 +107,7 @@ static inline bool is_valid_date(const date_t *date) {
     return date->month >= 1 && date->month <= 12 &&
            date->day >= 1 &&
            date->day <= (is_leap_year(date->year) &&
-           date->month == 1 ? 29 : days_in_month[date->month-1]);
+           date->month == 2 ? 29 : days_in_month[date->month-1]);
 }
 
 static time_t date_to_secs(const date_t *date) {
@@ -193,6 +198,31 @@ static inline bool eol(const date_t *date, const distro_t *distro) {
     ;
 }
 
+#ifdef DEBIAN
+static inline bool eol_lts(const date_t *date, const distro_t *distro) {
+    return !distro->milestones[MILESTONE_EOL_LTS] ||
+           date_ge(date, distro->milestones[MILESTONE_EOL_LTS])
+    ;
+}
+
+static inline bool eol_elts(const date_t *date, const distro_t *distro) {
+    return !distro->milestones[MILESTONE_EOL_ELTS] ||
+           date_ge(date, distro->milestones[MILESTONE_EOL_ELTS])
+    ;
+}
+#endif
+
+#ifdef UBUNTU
+static inline bool eol_esm(const date_t *date, const distro_t *distro) {
+    return distro->milestones[MILESTONE_EOL] &&
+           date_ge(date, distro->milestones[MILESTONE_EOL])
+           && (!distro->milestones[MILESTONE_EOL_ESM] ||
+              (distro->milestones[MILESTONE_EOL_ESM] &&
+               date_ge(date, distro->milestones[MILESTONE_EOL_ESM])))
+    ;
+}
+#endif
+
 // Filter callbacks
 
 static bool filter_all(unused(const date_t *date),
@@ -208,6 +238,23 @@ static bool filter_supported(const date_t *date, const distro_t *distro) {
     return created(date, distro) && !eol(date, distro);
 }
 
+#ifdef DEBIAN
+static bool filter_lts_supported(const date_t *date, const distro_t *distro) {
+    return created(date, distro) && eol(date, distro) && !eol_lts(date, distro);
+}
+
+static bool filter_elts_supported(const date_t *date, const distro_t *distro) {
+    return created(date, distro) && eol_lts(date, distro) && !eol_elts(date, distro);
+}
+#endif
+
+#ifdef UBUNTU
+static bool filter_esm_supported(const date_t *date, const distro_t *distro) {
+    return created(date, distro) && !eol_esm(date, distro) &&
+           strstr(distro->version, "LTS") != NULL;
+}
+#endif
+
 static bool filter_unsupported(const date_t *date, const distro_t *distro) {
     return created(date, distro) && eol(date, distro);
 }
@@ -242,6 +289,20 @@ static const distro_t *select_latest_release(const distro_elem_t *distro_list) {
     return selected;
 }
 
+static const distro_t *select_series(const distro_elem_t *distro_list, const char *series) {
+    const distro_t *selected;
+
+    selected = NULL;
+    while(distro_list != NULL) {
+        if(distro_list && (strcmp(distro_list->distro->series, series) == 0)) {
+            selected = distro_list->distro;
+            break;
+        }
+        distro_list = distro_list->next;
+    }
+    return selected;
+}
+
 static bool calculate_days(const distro_t *distro, const date_t *date,
                            int date_index, ssize_t *days) {
     const date_t *first;
@@ -358,8 +419,13 @@ static void free_data(distro_elem_t *list, char **content) {
         free(list->distro->milestones[MILESTONE_CREATED]);
         free(list->distro->milestones[MILESTONE_RELEASE]);
         free(list->distro->milestones[MILESTONE_EOL]);
+#ifdef DEBIAN
+        free(list->distro->milestones[MILESTONE_EOL_LTS]);
+        free(list->distro->milestones[MILESTONE_EOL_ELTS]);
+#endif
 #ifdef UBUNTU
         free(list->distro->milestones[MILESTONE_EOL_SERVER]);
+        free(list->distro->milestones[MILESTONE_EOL_ESM]);
 #endif
         free(list->distro);
         free(list);
@@ -383,9 +449,9 @@ static distro_elem_t *read_data(const char *filename, char **content) {
     data = *content = read_full_file(filename);
     line = strsep(&data, "\n");
     lineno = 1;
-    if(unlikely(strcmp(CSV_HEADER, line) != 0)) {
-        fprintf(stderr, NAME ": Header `%s' in file `%s' does not match "
-                "excatly `" CSV_HEADER "'.\n", line, filename);
+    if(unlikely(strncmp(CSV_HEADER, line, strlen(CSV_HEADER)) != 0)) {
+        fprintf(stderr, NAME ": Header `%s' in file `%s' does not start with "
+                "`" CSV_HEADER "'.\n", line, filename);
         failures++;
     }
 
@@ -495,6 +561,7 @@ static void print_help(void) {
            "Options:\n"
            "  -h  --help             show this help message and exit\n"
            "      --date=DATE        date for calculating the version (default: today)\n"
+           "      --series=SERIES    series to calculate the version for\n"
            "  -y[MILESTONE]          additionally, display days until milestone\n"
            "      --days=[MILESTONE] ("
           );
@@ -505,21 +572,28 @@ static void print_help(void) {
 
     printf(""
 #ifdef DEBIAN
-           "      --alias=DIST       print the alias (stable, testing, unstable) relative to\n"
-           "                         the distribution codename passed as an argument\n"
+           "      --alias=DIST       print the alias (oldstable, stable, testing, unstable)\n"
+           "                         relative to the given distribution codename\n"
 #endif
            "  -a  --all              list all known versions\n"
            "  -d  --devel            latest development version\n"
+#ifdef DEBIAN
+           "  -t  --testing          current testing version\n"
+#endif
+           "  -s  --stable           latest stable version\n"
 #ifdef UBUNTU
            "      --lts              latest long term support (LTS) version\n"
 #endif
 #ifdef DEBIAN
            "  -o  --oldstable        latest oldstable version\n"
 #endif
-           "  -s  --stable           latest stable version\n"
-           "      --supported        list of all supported stable versions\n"
+           "      --supported        list of all supported versions (including development)\n"
 #ifdef DEBIAN
-           "  -t  --testing          current testing version\n"
+           "  -l  --lts              list of all LTS supported versions\n"
+           "  -e  --elts             list of all Extended LTS supported versions\n"
+#endif
+#ifdef UBUNTU
+           "      --supported-esm    list of all Ubuntu Advantage supported stable versions\n"
 #endif
            "      --unsupported      list of all unsupported stable versions\n"
            "  -c  --codename         print the codename (default)\n"
@@ -535,13 +609,21 @@ static inline int not_exactly_one(void) {
             "--alias, "
 #endif
             "--all, --devel, "
-#ifdef UBUNTU
-            "--lts, "
+#ifdef DEBIAN
+            "--elts, "
 #endif
+#ifdef UBUNTU
+            "--latest, "
+#endif
+            "--lts, "
 #ifdef DEBIAN
             "--oldstable, "
 #endif
             "--stable, --supported, "
+#ifdef UBUNTU
+            "--supported-esm, "
+#endif
+            "--series, "
 #ifdef DEBIAN
             "--testing, "
 #endif
@@ -558,6 +640,7 @@ int main(int argc, char *argv[]) {
     const distro_t *selected;
     int i;
     int date_index = -1;
+    char *series_name = NULL;
     int option;
     int option_index;
     int return_value = EXIT_SUCCESS;
@@ -568,35 +651,45 @@ int main(int argc, char *argv[]) {
 #ifdef DEBIAN
     char *alias_codename = NULL;
 #endif
+#ifdef UBUNTU
+    bool filter_latest = false;
+#endif
 
     const struct option long_options[] = {
-        {"help",        no_argument,       NULL, 'h' },
-        {"date",        required_argument, NULL, 'D' },
-        {"all",         no_argument,       NULL, 'a' },
-        {"days",        optional_argument, NULL, 'y' },
-        {"devel",       no_argument,       NULL, 'd' },
-        {"stable",      no_argument,       NULL, 's' },
-        {"supported",   no_argument,       NULL, 'S' },
-        {"unsupported", no_argument,       NULL, 'U' },
-        {"codename",    no_argument,       NULL, 'c' },
-        {"fullname",    no_argument,       NULL, 'f' },
-        {"release",     no_argument,       NULL, 'r' },
+        {"help",          no_argument,       NULL, 'h' },
+        {"date",          required_argument, NULL, 'D' },
+        {"series",        required_argument, NULL, 'R' },
+        {"all",           no_argument,       NULL, 'a' },
+        {"days",          optional_argument, NULL, 'y' },
+        {"devel",         no_argument,       NULL, 'd' },
+        {"stable",        no_argument,       NULL, 's' },
+        {"supported",     no_argument,       NULL, 'S' },
+#ifdef UBUNTU
+        {"supported-esm", no_argument,       NULL, 'e' },
+#endif
+        {"unsupported",   no_argument,       NULL, 'U' },
+        {"codename",      no_argument,       NULL, 'c' },
+        {"fullname",      no_argument,       NULL, 'f' },
+        {"release",       no_argument,       NULL, 'r' },
 #ifdef DEBIAN
-        {"alias",       required_argument, NULL, 'A' },
-        {"oldstable",   no_argument,       NULL, 'o' },
-        {"testing",     no_argument,       NULL, 't' },
+        {"alias",         required_argument, NULL, 'A' },
+        {"elts",          no_argument,       NULL, 'e' },
+        {"lts",           no_argument,       NULL, 'l' },
+        {"oldstable",     no_argument,       NULL, 'o' },
+        {"testing",       no_argument,       NULL, 't' },
 #endif
 #ifdef UBUNTU
-        {"lts",         no_argument,       NULL, 'L' },
+        {"latest",        no_argument,       NULL, 'l' },
+        {"lts",           no_argument,       NULL, 'L' },
 #endif
-        {NULL,          0,                 NULL, '\0' }
+        {NULL,            0,                 NULL, '\0' }
     };
 
 #ifdef UBUNTU
-    const char *short_options = "hadscrfy::";
+    const char *short_options = "hadscrfly::";
 #endif
 #ifdef DEBIAN
-    const char *short_options = "hadscrfoty::";
+    const char *short_options = "hadescrfloty::";
 #endif
 
     // Suppress error messages from getopt_long
@@ -663,6 +756,22 @@ int main(int argc, char *argv[]) {
                 }
                 break;
 
+            case 'R':
+                // Only long option --series is used
+                if(unlikely(series_name != NULL)) {
+                    fprintf(stderr, NAME ": series requested multiple times.\n");
+                    return EXIT_FAILURE;
+                }
+                if(!optarg || !is_valid_codename(optarg)) {
+                    fprintf(stderr, NAME ": invalid distribution series `%s'\n",
+                            optarg);
+                    free(date);
+                    return EXIT_FAILURE;
+                }
+                selected_filters++;
+                series_name = optarg;
+                break;
+
             case 'f':
                 just_days = false;
                 print_cb = print_fullname;
@@ -674,6 +783,13 @@ int main(int argc, char *argv[]) {
                 return EXIT_SUCCESS;
 
 #ifdef UBUNTU
+            case 'l':
+                selected_filters++;
+                filter_latest = true;
+                filter_cb = filter_devel;
+                select_cb = select_latest_created;
+                break;
+
             case 'L':
                 // Only long option --lts is used
                 selected_filters++;
@@ -683,6 +799,18 @@ int main(int argc, char *argv[]) {
 #endif
 
 #ifdef DEBIAN
+            case 'e':
+                selected_filters++;
+                filter_cb = filter_elts_supported;
+                select_cb = NULL;
+                break;
+
+            case 'l':
+                selected_filters++;
+                filter_cb = filter_lts_supported;
+                select_cb = NULL;
+                break;
+
             case 'o':
                 selected_filters++;
                 filter_cb = filter_oldstable;
@@ -708,6 +836,15 @@ int main(int argc, char *argv[]) {
                 select_cb = NULL;
                 break;
 
+#ifdef UBUNTU
+            case 'e':
+                // Only long option --supported-esm is used
+                selected_filters++;
+                filter_cb = filter_esm_supported;
+                select_cb = NULL;
+                break;
+
+#endif
 #ifdef DEBIAN
             case 't':
                 selected_filters++;
@@ -748,6 +885,9 @@ int main(int argc, char *argv[]) {
                 } else if(optopt == 'D') {
                     fprintf(stderr, NAME ": option `--date' requires "
                             "an argument DATE\n");
+                } else if(optopt == 'R') {
+                    fprintf(stderr, NAME ": option `--series' requires "
+                            "an argument SERIES\n");
                 } else {
                     fprintf(stderr, NAME ": unrecognized option `-%c'\n",
                             optopt);
@@ -800,15 +940,20 @@ int main(int argc, char *argv[]) {
 
 #ifdef DEBIAN
     if(alias_codename) {
+        const distro_t *oldstable = get_distro(distro_list, date, filter_oldstable,
+                                               select_oldstable);
         const distro_t *stable = get_distro(distro_list, date, filter_stable,
                                             select_latest_release);
         const distro_t *testing = get_distro(distro_list, date, filter_testing,
                                              select_latest_created);
         const distro_t *unstable = get_distro(distro_list, date, filter_devel,
                                               select_first);
-        if(unlikely(stable == NULL || testing == NULL || unstable == NULL)) {
+        if(unlikely(oldstable == NULL || stable == NULL || testing == NULL ||
+                    unstable == NULL)) {
             fprintf(stderr, NAME ": " OUTDATED_ERROR "\n");
             return_value = EXIT_FAILURE;
+        } else if(strcmp(oldstable->series, alias_codename) == 0) {
+            printf("oldstable\n");
         } else if(strcmp(stable->series, alias_codename) == 0) {
             printf("stable\n");
         } else if(strcmp(testing->series, alias_codename) == 0) {
@@ -824,12 +969,25 @@ int main(int argc, char *argv[]) {
     }
 #endif
 
-    if(select_cb == NULL) {
+    if(select_cb == NULL && !series_name) {
         filter_data(distro_list, date, date_index, just_days, filter_cb, print_cb);
     } else {
-        selected = get_distro(distro_list, date, filter_cb, select_cb);
+        if(series_name) {
+            selected = select_series(distro_list, series_name);
+        } else {
+            selected = get_distro(distro_list, date, filter_cb, select_cb);
+#ifdef UBUNTU
+            if(selected == NULL && filter_latest) {
+                selected = get_distro(distro_list, date, filter_stable, select_latest_release);
+            }
+#endif
+        }
         if(selected == NULL) {
-            fprintf(stderr, NAME ": " OUTDATED_ERROR "\n");
+            if(series_name) {
+                fprintf(stderr, NAME ": unknown distribution series `%s'\n", series_name);
+            } else {
+                fprintf(stderr, NAME ": " OUTDATED_ERROR "\n");
+            }
             return_value = EXIT_FAILURE;
         } else {
             if(!print_cb(selected, date, date_index, just_days)) {
